/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var Book = {

	bookmarkService : Cc["@mozilla.org/browser/bookmarks-service;1"].getService(Ci.nsIBookmarksService),
	rdfService : Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService),
	dataSource : null,
	compositeDataSource : Cc["@mozilla.org/rdf/datasource;1?name=composite-datasource"].getService(Ci.nsIRDFCompositeDataSource),
	containerUtils : Cc["@mozilla.org/rdf/container-utils;1"].getService(Ci.nsIRDFContainerUtils),
	
	rdfLiteral : Ci.nsIRDFLiteral,
	rdfDate : Ci.nsIRDFDate,

	prop_addDate : "http://home.netscape.com/NC-rdf#BookmarkAddDate",
	prop_desc : "http://home.netscape.com/NC-rdf#Description",
	prop_lmDate : "http://home.netscape.com/WEB-rdf#LastModifiedDate",
	prop_lvDate : "http://home.netscape.com/WEB-rdf#LastVisitDate",
	prop_name : "http://home.netscape.com/NC-rdf#Name",
	prop_schedule : "http://home.netscape.com/WEB-rdf#Schedule",
	prop_surl : "http://home.netscape.com/NC-rdf#ShortcutURL",
	prop_url : "http://home.netscape.com/NC-rdf#URL",
	prop_status : "http://home.netscape.com/WEB-rdf#status",
	prop_icon : "http://home.netscape.com/NC-rdf#Icon",
	prop_livemarkurl : "http://home.netscape.com/NC-rdf#FeedURL",

	prop_type : "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
	prop_type_bookmark : "http://home.netscape.com/NC-rdf#Bookmark",
	prop_type_bookmarkSep : "http://home.netscape.com/NC-rdf#BookmarkSeparator",
	prop_type_Folder : "http://home.netscape.com/NC-rdf#Folder",
	prob_type_livemark : "http://home.netscape.com/NC-rdf#Livemark",

	init : function(){
		this.dataSource = this.rdfService.GetDataSource("rdf:bookmarks");
		this.bookmarkService.readBookmarks();
		this.compositeDataSource.AddDataSource(this.rdfService.GetDataSourceBlocking('rdf:bookmarks'));
		this.compositeDataSource.AddDataSource(this.rdfService.GetDataSourceBlocking('rdf:localsearch'));
	},
	
	getSize : function(){
		return this.getResourceArray().length;
	},
	
	getNode : function(url){
		var result = [];
		try {
			// match := rdf property
			// method := <contains, startswith, endswith>
			var refString = 'find:datasource=rdf:bookmarks&match=' + this.prop_url
						+ '&method=contains&text=' + encodeURIComponent(url);
			
			var res = this.rdfService.GetResource(refString);
			var pred = this.rdfService.GetResource('http://home.netscape.com/NC-rdf#child');
			var targetsEnumerator = this.compositeDataSource.GetTargets(res, pred, true);
		
			while(targetsEnumerator.hasMoreElements()) {
				var node = targetsEnumerator.getNext().QueryInterface(Ci.nsIRDFResource);

				//minimal requirements for a valid bookmark
				var target_name = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_name), true);
				var target_url = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_url), true);

				if (!(target_name && target_url))
					continue;
				
				if(!StringUtils.isURL(this.getURL(node)))
					continue;

				result.push(node);
			}
		} catch(e) {
			alert(e);
		}
		return result;
	},
	
	getParent : function(object){
		var nodes = [];
		if(ObjectUtils.isRDFResource(object)){
			nodes.push(object);
		}else if(ObjectUtils.isString(object) && StringUtils.isURL(object)){
			nodes = this.getNode(object);
		}
		
		if(nodes.length > 0){
			var aResource = nodes[0];
			var arcsIn = this.dataSource.ArcLabelsIn(aResource);
			
			while (arcsIn.hasMoreElements()) {
				var containerArc = arcsIn.getNext();
				// Test if the arc is ordinal. We will assume that the parent is determined
				// by the first ordinal arc that points to the resource.
				// That's not bullet-proof, since several ordinal arcs can point to the same
				// resource. However, this will be sufficient for most of the cases.
				if (this.containerUtils.IsOrdinalProperty(containerArc)) {
					return this.dataSource.GetSource(containerArc, aResource, true).QueryInterface(Ci.nsIRDFResource);
				}
			}

		}
		return null;
	},
	
	getParentContainer : function(resource){
		var parent = this.getParent(resource);
		var container = Cc["@mozilla.org/rdf/container;1"].createInstance(Ci.nsIRDFContainer);
		if(parent && parent != null)
			container.Init(this.dataSource, parent);
			
		return container;
	},
	
	removeBookmark : function(resource){
		try {
			var container = this.getParentContainer(resource);
			this.dataSource.beginUpdateBatch();
			container.RemoveElement(resource, true);
			this.dataSource.endUpdateBatch();
		} catch(e) {
			alert(e);
		}
	},
	
	getResourceArray : function(){
		return this.fill([]);
	},
	
	getResourceHash : function(){ 
		return this.fill(new ExtendedHash());
	},
	
	getResourceIDHash : function(){
		return this.fill(new ExtendedHash(), 'ID');
	},
	
	fillRecursive : function(node, object, id){
	
		if(this.containerUtils.IsContainer(this.dataSource, node)){
		
			var container = Cc["@mozilla.org/rdf/container;1"].createInstance(Ci.nsIRDFContainer);
			container.Init(this.dataSource, node);
			
			var e = container.GetElements();
			while(e.hasMoreElements()) {
				var next = e.getNext().QueryInterface(Ci.nsIRDFResource);
				this.fillRecursive(next, object, id);
			}
			
		}else {
			//minimal requirements for a valid bookmark
			var target_name = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_name), true);
			var target_url = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_url), true);

			if (!(target_name && target_url))
				return;
			
			if(!StringUtils.isURL(this.getURL(node)))
				return;

			this.put(object, node, id);
		}
	
	},
	
	fill : function(object, id){
		var rootnode = this.rdfService.GetResource('NC:BookmarksRoot');
		this.dataSource = this.dataSource.QueryInterface(Ci.nsIRDFDataSource);

		this.fillRecursive(rootnode, object, id);
	
  		return object;
	},
	
	put : function(object, node, id){
		if(ObjectUtils.isArray(object)){
			object.push(node);
		}else {
			if(object instanceof ExtendedHash){
				if(id){
					object.put(node.Value, node); //TODO replace it with real resource id
				}else {
					object.put(this.getURI(node), node);
				}
			}else {
				if(ObjectUtils.isFunction(object)){
					object(node);
				}else {
					throw 'Unknown base type : (' + (typeof object) + ')';
				}
			}
		}
	},
	
	isLivemark : function(node){
		var type = this.getType(node);
		if(type == this.prob_type_livemark){
			return true;
		}
		return false;
	},
	
	isBookmarked : function(url){
		return ArrayUtils.isNotEmpty(this.getNode(url));
	},
	
	getType : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_type), true);
		if(target){
			return target.QueryInterface(Ci.nsIRDFResource).Value;
		}else {
			if(StringUtils.isURL(this.getURL(node))){
	            return this.prop_type_bookmark;
	        }
		}
		return null;
	},

	getName : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_name), true);
        if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	},

	getURL : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_url), true);
        if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	},

	getFURL : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_livemarkurl), true);
        if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	},
	
	getURI : function(node){
		if(this.isLivemark(node)){
			return this.getFURL(node);
		}
		return this.getURL(node);
	},

	getKeys : function(node){
    	var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_surl), true);
    	if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	},

	getAddDate : function(node){
    	var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_addDate), true);
    	if(target){
			return target.QueryInterface(this.rdfDate).Value;
		}
		return StringUtils.EMPTY;
	},

	getLMDate : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_lmDate), true);
    	if(target){
			return target.QueryInterface(this.rdfDate).Value;
		}
		return StringUtils.EMPTY;
	},

	getLVDate : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_lvDate), true);
    	if(target){
			return target.QueryInterface(this.rdfDate).Value;
		}
		return StringUtils.EMPTY;
	},

	getDesc : function(node){
		var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_desc), true);
        if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	},

	getIcon : function(node){
    	var target = this.dataSource.GetTarget(node, this.rdfService.GetResource(this.prop_icon), true);
        if(target){
			return target.QueryInterface(this.rdfLiteral).Value;
		}
		return StringUtils.EMPTY;
	}

};

Book.init();
